class Canvas
  require_relative 'dot'
  require 'cjk_helper'

  attr_accessor :height
  attr_accessor :width
  attr_accessor :dots

  def initialize(height:, width:, frame_style:)
    if !height.is_a?(Fixnum) || !width.is_a?(Fixnum)
      raise 'FATAL: height and width must given and should be number'
    end

    @height = height
    @width = width

    if frame_style == 'dotted'
      init_canvas_with_style(height: @height, width: @width, symbol:'.')
    elsif frame_style == 'star'
      init_canvas_with_style(height: @height, width: @width, symbol:'*')
    else
      @dots = Array.new(height) { Array.new(width, nil) }

      (0...@height).each do |j|
        (0...@width).each do |i|
          @dots[j][i] = Dot.new(x: i, y: j, symbol:nil)
        end
      end
    end 
  end

  def init_canvas_with_style(height:, width:, symbol:)
    @dots = Array.new(height) { Array.new(width, nil) }

    (0...@height).each do |j|
      (0...@width).each do |i|
        if i == 0 || i == @width - 1 || j == 0 || j == @height - 1
          @dots[j][i] = Dot.new(x: i, y: j, symbol:symbol)
          @dots[j][i].occupied = true
        else
          @dots[j][i] = Dot.new(x: i, y: j, symbol:nil)
        end
      end
    end
  end

  def draw(y:, x:, symbol:)
    if !y.is_a?(Fixnum) || !x.is_a?(Fixnum)
      raise 'FATAL: x and y must given and should be number'
    end

    if y >= @height
      raise 'FATAL: y should not be bigger than canvas height'
    end 

    if x >= @width
      raise 'FATAL: x should not be bigger than canvas width'
    end
    
    if @dots[y][x].occupied
      raise "FATAL: this dot(#{x}, #{y}) is occupied"
    end

    # ++
    # CJK character takes two-char width on terminal print
    # so we need take care of it especailly
    # ++
    if CJKHelper.is_cjk(symbol)
      if x + 1 < @width
        @dots[y][x].symbol = symbol
        @dots[y][x + 1].occupied = true
        @dots[y][x + 1].symbol = nil
      else
        @dots[y+1][0].symbol = symbol
        @dots[y+1][1].occupied = true
        @dots[y+1][1].symbol = nil
      end
    else
      @dots[y][x].symbol = symbol
    end
  end

  def show
    canvas = ''
    (0...@height).each do |i|
      canvas += "\n"
      (0...@width).each do |j|
        if !@dots[i][j].occupied && @dots[i][j].symbol == nil
        canvas += ' '
        else
          canvas += @dots[i][j].symbol.to_s
        end
      end
    end
    puts canvas
  end

  def occupied?(x:, y:)
    @dots[y][x].occupied
  end
end
