require_relative 'canvas'

class Card
  require 'cjk_helper'

  attr_reader :canvas

  def initialize(title:, sub_title:, note:, from:)
    @title = title
    @sub_title = sub_title
    @note = note
    @from = from

    @canvas = Canvas.new(height: 10, width: 62, frame_style:'dotted')

    if @title.length < 60
      cjk_in_title = CJKHelper.get_cjk(@title)
      @title_x_start = (60 - @title.length - cjk_in_title.length)/2 
    else
      raise "title too long"
    end

    if @sub_title.length < 60
      cjk_in_sub_title = CJKHelper.get_cjk(@sub_title)
      @sub_title_x_start = (60 - @sub_title.length - cjk_in_sub_title.length)/2
    else
      raise "sub title too long"
    end

    init_title
    init_sub_title
    init_note
    init_from
  end

  def init_title
    y = 1
    x = @title_x_start
    @title.chars.each do |c|
      while @canvas.occupied?(x: x, y: y)
        x += 1 if x < 60
      end
      @canvas.draw(y: y, x: x, symbol: c)
      x += 1
    end
  end

  def init_sub_title
    y = 2
    x = @sub_title_x_start
    @sub_title.chars.each do |c|
      while @canvas.occupied?(x: x, y: y)
        x += 1 if x < 60
      end

      @canvas.draw(y: y, x: x, symbol: c)
      x += 1
    end
  end

  def init_note
    return if !@note
    y = 4
    x = 2
    @note.chars.each do |c|
      if x <= 60
        while @canvas.occupied?(y:y, x:x)
          x += 1 if x < 60
        end
        @canvas.draw(y: y, x: x, symbol: c)
      else
        y += 1
        x = 1
      end
      x += 1
    end
  end

  def init_from
    y = 10 - 2
    x = 60 - @from.length
    @from.chars.each do |c|
      while @canvas.occupied?(x: x, y: y)
        x += 1 if x < 60
      end

      @canvas.draw(y: y, x: x, symbol: c)
      x += 1
    end
  end

  def show
    @canvas.show
  end
end
